# encoding: utf-8

import re
import urllib2
from BeautifulSoup import BeautifulSoup
from contextlib import closing
from urlparse import urljoin

class OpenGraph(object):
    """
    """

    
    scrape = False

    def __init__(self, url="http://example.com", html=None, scrape=False, required_attrs = set(('title', 'type', 'image', 'url')), **kwargs):
        # If scrape == True, then will try to fetch missing attribtues
        # from the page's body
        self.scrape = scrape
        self.url = url
        self.items = {}
        self.required_attrs = set(required_attrs)
            
        if not html:
            with closing(urllib2.urlopen(url)) as raw:
                html = raw.read()
        
        self.parser(html)
    
    def absolute(self, url):
        return urljoin(self.url, url)
    
    def parser(self, html):
        """
        """
        if not isinstance(html,BeautifulSoup):
            doc = BeautifulSoup(html)
        else:
            doc = html
        ogs = doc.html.head.findAll(property=re.compile(r'^og'))
        for og in ogs:
            if og.has_key(u'content'):
                self.items[og[u'property'][3:]]=og[u'content']
        
        # Couldn't fetch all attrs from og tags, try scraping body
        if self.scrape:
            remaining_keys = self.required_attrs - set(self.items.viewkeys())
            for key in remaining_keys:
                self.items[key] = getattr(self, 'scrape_{key}'.format(key=key))(doc)
        
        image = self.items.get("image", False)
        if image:
            self.items["image"] = self.absolute(self.items["image"])
        
    def is_valid(self):
        return self.required_attrs <= set(self.items.viewkeys())
        
    def to_html(self):
        if not self.is_valid():
            return u"<meta property=\"og:error\" content=\"og metadata is not valid\" />"
            
        meta = u""
        for key,value in self.items.iteritems():
            meta += u"\n<meta property=\"og:%s\" content=\"%s\" />" %(key, value)
        meta += u"\n"
        
        return meta
        
    def scrape_image(self, doc):
        images = [dict(img.attrs)['src'] 
            for img in doc.html.body.findAll('img')]

        if images:
            return images[0]

        return u''

    def scrape_title(self, doc):
        return doc.html.head.title.text

    def scrape_type(self, doc):
        return 'other'

    def scrape_url(self, doc):
        return self.url
    
    def scrape_description(self, doc):
        ogs = doc.html.head.findAll(name='meta', attrs={"name":("description", "DC.description", "eprints.abstract")}, )
        for og in ogs:
            content = og.get("content", False)
            if content:
                return content
        else:
            heading = doc.html.find(re.compile("h[1-6]"))
            if heading:
                return heading.text
            else:
                return doc.html.find("p").text
        



# encoding: utf-8

import unittest
import opengraph


HTML = """
<html xmlns:og="http://ogp.me/ns#">
<head>
<title>The Rock (1996)</title>
<meta property="og:title" content="The Rock" />
<meta property="og:type" content="movie" />
<meta property="og:url" content="http://www.imdb.com/title/tt0117500/" />
<meta property="og:image" content="http://ia.media-imdb.com/images/rock.jpg" />
</head>
</html>
"""

class test(unittest.TestCase):
    def test_url(self):
        data = opengraph.OpenGraph(url='http://vimeo.com/896837')
        self.assertEqual(data.items['url'], 'http://vimeo.com/896837')
        
    def test_isinstace(self):
        data = opengraph.OpenGraph()
        self.assertTrue(isinstance(data,opengraph.OpenGraph))
        
    def test_to_html(self):
        og = opengraph.OpenGraph(html=HTML)
        self.assertTrue(og.to_html())
        
    def test_is_valid(self):
        og = opengraph.OpenGraph(url='http://grooveshark.com')
        self.assertTrue(og.is_valid())

    def test_is_not_valid(self):
        og = opengraph.OpenGraph(url='http://vdubmexico.com')
        self.assertFalse(og.is_valid())
    
    def test_required(self):
        og = opengraph.OpenGraph(url='http://grooveshark.com', required_attrs=("description",), scrape=True)
        self.assertTrue(og.is_valid())
    
    def test_scrape(self):
        og = opengraph.OpenGraph(url='http://graingert.co.uk/', required_attrs=("description",), scrape=True)
        self.assertTrue(og.is_valid())
        self.assertTrue(og.items["description"])
        
        og = opengraph.OpenGraph(url='http://www.crummy.com/software/BeautifulSoup/bs3/documentation.html', required_attrs=("description",), scrape=True)
        self.assertEqual(og.items["description"], "Beautiful Soup Documentation")

    
    def test_absolute(self):
        og = opengraph.OpenGraph(url='http://www.crummy.com/software/BeautifulSoup/bs3/documentation.html', required_attrs=("image",), scrape=True)
        self.assertEqual(og.items["image"], "http://www.crummy.com/software/BeautifulSoup/bs3/6.1.jpg")

    
if __name__ == '__main__':
    unittest.main()



from os.path import join, dirname
from distutils.core import setup

try:
    f = open(join(dirname(__file__), 'README.rst'))
    long_description = f.read().strip()
    f.close()
except IOError:
    long_description = ""


version = '0.9.1'

setup(
    name='ogp',
    version=version,
    description="A module to parse the Open Graph Protocol",
    long_description=long_description + "\n",
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Topic :: Text Processing :: Markup :: HTML',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
    keywords='opengraph protocol facebook',
    author='Erik Rivera',
    author_email='erik.river@gmail.com',
    url='https://github.com/graingert/opengraph',
    license='MIT',
    packages=["ogp",],
    install_requires=['BeautifulSoup'],
)
    
